# Directed Reachability Benchmarks

This data set contains a set of benchmark instances,
which were used to evaluate
the TACAS'21 submission 
"Directed Reachability for Infinite-State Systems" (preprint available [online](https://arxiv.org/abs/2010.07912) **[BHO20]**).
Additionally, we include an original, larger benchmark suite
containing safe and unsafe coverability instances,
of which we used only the unsafe subset in the evaluation
given in the paper.

The folder `nets` contains the instances that were used for benchmarking
our tool, while the folder `original_nets` contains
the original safe and unsafe coverability instances.

## Benchmark Suites

### `original_nets`

The instances in `original_nets` were
obtained from the tool **QCover**,
available online [here](https://github.com/blondimi/qcover).
Originally, these instances 
come from 5 sources:
- `bug_tracking`: from an issue tracking system **[J09]**
- `medical`: from analysis in the context of medical messaging **[BMDS07]**

- `mist_PN` and `mist_boundedPN`: from mutual exclusion
algorithms and communication protocols **[G02]**

- `soter`: from the verification of ERLANG concurrent programs **[DKO13]**

- `wahl-kroening`: from the analysis of multi-threaded
C programs with shared memory **[KKW13]**

### `nets`

In `nets`, there are three benchmark suites.
Each suite is also present in a pre-pruned form, where
we have applied sign analysis **[GLS18]** in order
to remove places and transitions that are guaranteed to never be marked (for places)
or fired (for transitions) by any run
reaching a target.

The benchmark suites are obtained as follows:
- `coverability`: by keeping only unsafe instances from the benchmark suites in `original_nets`
- `random_walk`: obtained by us by performing random walks on
the largest 25% of nets for each benchmark suite in `original_nets`. These random walks are performed by starting at the initial marking specified by the instance,
then applying a number of (fireable) transitions, where the choice is uniformly at random from among enabled transitions. The resulting marking becomes the target for a reachability query. We performed random walks of lengths 20, 25, 30, 35, 40, 50, 60, 75, 90 and 100. Finally, we removed all instances where
any tool was able to give a witness of length 20 or less, in order to keep only challenging instances and reduce the noise of trivial instances.
The length of the random walk is indicated in the file name, as is the original instance. Let us, as an example, consider the instance `dekker_vs_satabs.2_multi_25_0.lola`. It uses the underlying net from the instance `dekker_vs_satabs.2`, and used a random walk of length 25. `multi` indicates that, if the original instance has an upward-closet set as potential initial markings, we start with a random number of tokens between 1 and 20% of the length of the random walk.
- `sypet`: obtained by us by extracting queries from program synthesis, where Petri nets are used to find type-checking programs **[FMWDR17]**.

For statistical information
about the size distribution, see the TACAS'21 publication or its preprint **[BHO20]**.

## File Formats

Due to the many sources the set of benchmarks is compiled from, and the many tools
that were evaluated on it, it contains a variety of file formats.

- `.lola` and `.formula`: The formats for
nets and targets for the
tool LoLA, see e.g. **[W18]**
- `.spec`: The format for the tool `mist` (among others), see 
https://github.com/pierreganty/mist/wiki for a description of the format.
- `.tts` and `.prop`: The formats for nets and targets for the tool `bfc`, see http://www.cprover.org/bfc/ for a description of the format.
- `.pl`: Not used by any tool evaluated by us, and included only for completeness.

## Questions
For any questions or comments concerning this dataset, contact [Philip Offtermatt](https://p-offtermatt.github.io/). 

## References

**[BFHH16]** [Michael Blondin, Alain Finkel, Christoph Haase and Serge Haddad. *Approaching the Coverability Problem Continuously*. Proc. 22<sup>nd</sup> International Conference Tools and Algorithms for the Construction and Analysis of Systems (TACAS), Springer, 2016](http://dx.doi.org/10.1007/978-3-662-49674-9_28). Available online [here](http://info.usherbrooke.ca/mblondin/papers/BFHH16.pdf).

**[J09]** Jan ́ak, J.: Issue Tracking Systems. Master’s thesis, Masaryk University (2009)

**[BMDS07]** [Barth,   A.,   Mitchell,   J.C.,   Datta,   A.,   Sundaram,   S.:   Privacy   and   util-ity   in   business   processes.   In:   Proc.   20thIEEE   Computer   Security   Foun-dations   Symposium   (CSF).   pp.   279–294.   IEEE   Computer   Society   (2007)](https://doi.org/10.1109/CSF.2007.26)

**[G02]**  Ganty, P.: Algorithmes et structures de donn ́ees efficaces pour la manipulation decontraintes sur les intervalles. Master’s thesis, Universit ́e Libre de Bruxelles (2002),(In French)

**[DKO13]** [Automatic verification of erlang-style con-currency. In: Proc. 20thInternational Symposium on Static Analysis (SAS). pp.454–476. Springer (2013).](https://doi.org/10.1007/978-3-642-38856-9_24)

**[KKW13]**  [Kaiser, A., Kroening, D., Wahl, T.: A widening approach to multithreaded pro-gram  verification.  ACM  Transactions  on  Programming  Languages  and  Systems(TOPLAS)36(4), 14:1–14:29 (2014).](https://doi.org/10.1145/2629608)

**[GLS18]** [Thomas Geffroy, Jérôme Leroux, Grégoire Sutre:
Occam's Razor applied to the Petri net coverability problem.
In:
Theoretical Computer Science (TCS).
Volume 750.
pp.38-52. (2018)](https://doi.org/10.1016/j.tcs.2018.04.014)

 **[FMWDR17]** [Feng,  Y.,  Martins,  R.,  Wang,  Y.,  Dillig,  I.,  Reps,  T.W.:  Component-basedsynthesis  for  complex  APIs.  In:  Proc.  44thACM  SIGPLAN  Symposium  onPrinciples   of   Programming   Languages   (POPL).   pp.   599–612.   ACM   (2017).](https://doi.org/10.1145/3009837.3009851)

 **[BHO20]** [Blondin, M., Haase, C., & Offtermatt, P. (2020). Directed Reachability for Infinite-State Systems. ArXiv, abs/2010.07912.](https://arxiv.org/abs/2010.07912)

 **[W18]** [Wolf K. (2018) Petri Net Model Checking with LoLA 2. In: Khomenko V., Roux O. (eds) Application and Theory of Petri Nets and Concurrency. PETRI NETS 2018. Lecture Notes in Computer Science, vol 10877. Springer, Cham.](https://doi.org/10.1007/978-3-319-91268-4_18)